@@ -1,5 +1,5 @@
 package App::week;
-our $VERSION = "1.0205";
+our $VERSION = "1.03";
 
 use v5.14;
 use warnings;
@@ -25,6 +25,7 @@ my @DOW_LABELS = qw(
     DOW_TH
     DOW_FR
     DOW_SA
+    DOW_CW
     );
 
 my %DEFAULT_COLORMAP = (
@@ -52,43 +53,49 @@ use Getopt::EX::Hashed; {
     has mday => default => $mday;
     has mon  => default => $mon + 1;
 
-    has cell_width   => default => 22;
+    has cell_width   => default => undef;
     has frame        => default => '  ';
     has frame_height => default => 1;
 
     # option params
-    has help        => spec => ' h        ' ;
-    has version     => spec => ' v        ' ;
-    has months      => spec => ' m =i     ' , default => 0;
-    has after       => spec => ' A :1     ' , min => 0;
-    has before      => spec => ' B :1     ' , min => 0, default => 1;
-    has center      => spec => ' C :4     ' , min => 0;
-    has column      => spec => ' c =i     ' , min => 1, default => 3;
-    has colordump   => spec => '          ' ;
-    has colormap    => spec => '   =s@ cm ' , default => [];
-    has show_year   => spec => ' y        ' ;
-    has years       => spec => ' Y :1     ' , max => 100;
-    has rgb24       => spec => '   !      ' ;
-    has year_on_all => spec => ' P        ' ;
-    has year_on     => spec => ' p =i     ' , min => 0, max => 12;
-    has config      => spec => '   =s%    ' , default => {};
+    has help        => ' h        ' ;
+    has version     => ' v        ' ;
+    has months      => ' m =i     ' , default => 0;
+    has after       => ' A :1     ' , min => 0;
+    has before      => ' B :1     ' , min => 0, default => 1;
+    has center      => ' C :4     ' , min => 0;
+    has column      => ' c =i     ' , min => 1, default => 3;
+    has colordump   => '          ' ;
+    has colormap    => '   =s@ cm ' , default => [];
+    has show_year   => ' y        ' ;
+    has years       => ' Y :1     ' , max => 100;
+    has rgb24       => '   !      ' ;
+    has year_on_all => ' P        ' ;
+    has year_on     => ' p =i     ' , min => 0, max => 12;
+    has config      => '   =s%    ' , default => {};
+    has weeknumber  => ' W :1     ' ;
 
-    has '+center' =>
-	action => sub { $_->{after} = $_->{before} = $_[1] };
+    has '+center' => sub {
+	$_->{after} = $_->{before} = $_[1];
+    };
 
-    has '+help' => action => sub {
+    has '+weeknumber' => sub {
+	${$_->config}{weeknumber} = $_[1];
+    };
+
+    has '+help' => sub {
 	pod2usage
 	    -verbose  => 99,
 	    -sections => [ qw(SYNOPSIS VERSION) ];
     };
 
-    has '+version' => action  => sub {
+    has '+version' => sub {
 	print "Version: $VERSION\n";
 	exit;
     };
 
     has "<>" =>
-	action => sub {
+	sub {
 	    my $obj = $_;
 	    local $_ = $_[0];
 	    if (/^-+([0-9]+)$/) {
@@ -138,7 +145,7 @@ sub argv {
     my $app = shift;
     for (@{$app->ARGV}) {
 	call \&guess_date,
-	    for => $app,
+	    for  => $app,
 	    with => [ qw(year mon mday show_year) ];
     }
     return $app;
@@ -290,6 +297,9 @@ sub cell {
 
     my @cal = @{$calyear[$y][$m]};
 
+    # XXX this is not the best place to initialize...
+    $obj->{cell_width} //= length $cal[2];
+
     my %label;
     @label{qw(month week days)} = $d
 	? qw(THISMONTH THISWEEK THISDAYS)
@@ -298,9 +308,9 @@ sub cell {
     $cal[0] = $obj->color($label{month}, $cal[0]);
     $cal[1] = $obj->color($label{week},
 			  state $week = $obj->week_line($cal[1]));
-    my $day_re = $d ? qr/${\(sprintf '%2d', $d)}\b/ : undef;
+    my $day_re = $d ? qr/^(?: [ \d]{2}){0,6} \K(${\(sprintf '%2d', $d)})\b/ : undef;
     for (@cal[ 2 .. $#cal ]) {
-	s/($day_re)/$obj->color("THISDAY", $1)/e if $day_re;
+	s/$day_re/$obj->color("THISDAY", $1)/e if $day_re;
 	$_ = $obj->color($label{days}, $_);
     }
 
@@ -311,9 +321,10 @@ sub week_line {
     my $obj = shift;
     my $week = shift;
     my @week = split_week $week;
-    for (0..6) {
+    for (0..7) {
 	if (my $color = $obj->COLORMAP->{$DOW_LABELS[$_]}) {
 	    my $i = $_ * 2 + 1;
+	    $i > $#week and last;
 	    $week[$i] = $obj->color($color, $week[$i]);
 	}
     }
@@ -328,12 +339,16 @@ __END__
 
 =head1 NAME
 
-week - colorful calendar command
+week - colorful calendar command for ANSI terminal
 
 =head1 SYNOPSIS
 
 B<week> [ -MI<module> ] [ option ] [ date ]
 
+=head1 VERSION
+
+Version 1.03
+
 =head1 DESCRIPTION
 
 Yet another calendar command.  Read the script's manual for detail.
